# Copyright(c) 2018 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack import exceptions
from openstack.instance_ha.v1 import host as _host
from openstack.instance_ha.v1 import notification as _notification
from openstack.instance_ha.v1 import segment as _segment
from openstack.instance_ha.v1 import vmove as _vmove
from openstack import proxy
from openstack import resource


class Proxy(proxy.Proxy):
    """Proxy class for ha resource handling.

    Create method for each action of each API.
    """

    _resource_registry = {
        "host": _host.Host,
        "notification": _notification.Notification,
        "segment": _segment.Segment,
        "vmove": _vmove.VMove,
    }

    def notifications(self, **query):
        """Return a generator of notifications.

        :param kwargs query: Optional query parameters to be sent to
            limit the notifications being returned.
        :returns: A generator of notifications
        """
        return self._list(_notification.Notification, **query)

    def get_notification(self, notification):
        """Get a single notification.

        :param notification: The value can be the ID of a notification or a
            :class:`~masakariclient.sdk.ha.v1.notification.Notification`
            instance.
        :returns: One
            :class:`~masakariclient.sdk.ha.v1.notification.Notification`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_notification.Notification, notification)

    def create_notification(self, **attrs):
        """Create a new notification.

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`masakariclient.sdk.ha.v1.notification.Notification`,
            comprised of the propoerties on the Notification class.
        :returns: The result of notification creation
        :rtype: :class:`masakariclient.sdk.ha.v1.notification.Notification`
        """
        return self._create(_notification.Notification, **attrs)

    def segments(self, **query):
        """Return a generator of segments.

        :param kwargs query: Optional query parameters to be sent to
            limit the segments being returned.
        :returns: A generator of segments
        """
        return self._list(_segment.Segment, **query)

    def get_segment(self, segment):
        """Get a single segment.

        :param segment: The value can be the ID of a segment or a
            :class:`~masakariclient.sdk.ha.v1.segment.Segment` instance.
        :returns: One :class:`~masakariclient.sdk.ha.v1.segment.Segment`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_segment.Segment, segment)

    def create_segment(self, **attrs):
        """Create a new segment.

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`masakariclient.sdk.ha.v1.segment.Segment`,
            comprised of the propoerties on the Segment class.
        :returns: The result of segment creation
        :rtype: :class:`masakariclient.sdk.ha.v1.segment.Segment`
        """
        return self._create(_segment.Segment, **attrs)

    def update_segment(self, segment, **attrs):
        """Update a segment.

        :param segment: The value can be the ID of a segment or a
            :class:`~masakariclient.sdk.ha.v1.segment.Segment` instance.
        :param dict attrs: Keyword arguments which will be used to update
            a :class:`masakariclient.sdk.ha.v1.segment.Segment`,
            comprised of the propoerties on the Segment class.
        :returns: The updated segment.
        :rtype: :class:`masakariclient.sdk.ha.v1.segment.Segment`
        """
        return self._update(_segment.Segment, segment, **attrs)

    def delete_segment(self, segment, ignore_missing=True):
        """Delete a segment.

        :param segment:
            The value can be either the ID of a segment or a
            :class:`~masakariclient.sdk.ha.v1.segment.Segment` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the segment does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent segment.
        :returns: ``None``
        """
        return self._delete(
            _segment.Segment, segment, ignore_missing=ignore_missing
        )

    def hosts(self, segment_id, **query):
        """Return a generator of hosts.

        :param segment_id: The ID of a failover segment.
        :param kwargs query: Optional query parameters to be sent to
            limit the hosts being returned.

        :returns: A generator of hosts
        """
        return self._list(_host.Host, segment_id=segment_id, **query)

    def create_host(self, segment_id, **attrs):
        """Create a new host.

        :param segment_id: The ID of a failover segment.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`masakariclient.sdk.ha.v1.host.Host`,
            comprised of the propoerties on the Host class.

        :returns: The results of host creation
        """
        return self._create(_host.Host, segment_id=segment_id, **attrs)

    def get_host(self, host, segment_id=None):
        """Get a single host.

        :param segment_id: The ID of a failover segment.
        :param host: The value can be the ID of a host or a :class:
            `~masakariclient.sdk.ha.v1.host.Host` instance.

        :returns: One :class:`~masakariclient.sdk.ha.v1.host.Host`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        :raises: :class:`~openstack.exceptions.InvalidRequest`
            when segment_id is None.
        """
        if segment_id is None:
            raise exceptions.InvalidRequest("'segment_id' must be specified.")

        host_id = resource.Resource._get_id(host)
        return self._get(_host.Host, host_id, segment_id=segment_id)

    def update_host(self, host, segment_id, **attrs):
        """Update the host.

        :param segment_id: The ID of a failover segment.
        :param host: The value can be the ID of a host or a :class:
            `~masakariclient.sdk.ha.v1.host.Host` instance.
        :param dict attrs: The attributes to update on the host represented.

        :returns: The updated host
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        :raises: :class:`~openstack.exceptions.InvalidRequest`
            when segment_id is None.
        """
        host_id = resource.Resource._get_id(host)
        return self._update(
            _host.Host, host_id, segment_id=segment_id, **attrs
        )

    def delete_host(self, host, segment_id=None, ignore_missing=True):
        """Delete the host.

        :param segment_id: The ID of a failover segment.
        :param host: The value can be the ID of a host or a :class:
            `~masakariclient.sdk.ha.v1.host.Host` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the host does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent host.

        :returns: ``None``
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        :raises: :class:`~openstack.exceptions.InvalidRequest`
            when segment_id is None.

        """
        if segment_id is None:
            raise exceptions.InvalidRequest("'segment_id' must be specified.")

        host_id = resource.Resource._get_id(host)
        return self._delete(
            _host.Host,
            host_id,
            segment_id=segment_id,
            ignore_missing=ignore_missing,
        )

    def vmoves(self, notification, **query):
        """Return a generator of vmoves.

        :param notification: The value can be the UUID of a notification or
            a :class: `~masakariclient.sdk.ha.v1.notification.Notification`
            instance.
        :param kwargs query: Optional query parameters to be sent to
            limit the vmoves being returned.

        :returns: A generator of vmoves
        """
        notification_id = resource.Resource._get_id(notification)
        return self._list(
            _vmove.VMove,
            notification_id=notification_id,
            **query,
        )

    def get_vmove(self, vmove, notification):
        """Get a single vmove.

        :param vmove: The value can be the UUID of one vmove or
            a :class: `~masakariclient.sdk.ha.v1.vmove.VMove` instance.
        :param notification: The value can be the UUID of a notification or
            a :class: `~masakariclient.sdk.ha.v1.notification.Notification`
            instance.
        :returns: one 'VMove' resource class.
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        :raises: :class:`~openstack.exceptions.InvalidRequest`
            when notification_id is None.
        """
        notification_id = resource.Resource._get_id(notification)
        vmove_id = resource.Resource._get_id(vmove)
        return self._get(
            _vmove.VMove,
            vmove_id,
            notification_id=notification_id,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.identity.v2 import extension as _extension
from openstack.identity.v2 import role as _role
from openstack.identity.v2 import tenant as _tenant
from openstack.identity.v2 import user as _user
from openstack import proxy


class Proxy(proxy.Proxy):
    def extensions(self):
        """Retrieve a generator of extensions

        :returns: A generator of extension instances.
        :rtype: :class:`~openstack.identity.v2.extension.Extension`
        """
        return self._list(_extension.Extension)

    def get_extension(self, extension):
        """Get a single extension

        :param extension: The value can be the ID of an extension or a
            :class:`~openstack.identity.v2.extension.Extension`
            instance.

        :returns: One :class:`~openstack.identity.v2.extension.Extension`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no extension can be found.
        """
        return self._get(_extension.Extension, extension)

    def create_role(self, **attrs):
        """Create a new role from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.identity.v2.role.Role`,
            comprised of the properties on the Role class.

        :returns: The results of role creation
        :rtype: :class:`~openstack.identity.v2.role.Role`
        """
        return self._create(_role.Role, **attrs)

    def delete_role(self, role, ignore_missing=True):
        """Delete a role

        :param role: The value can be either the ID of a role or a
            :class:`~openstack.identity.v2.role.Role` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the role does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent role.

        :returns: ``None``
        """
        self._delete(_role.Role, role, ignore_missing=ignore_missing)

    def find_role(self, name_or_id, ignore_missing=True):
        """Find a single role

        :param name_or_id: The name or ID of a role.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.identity.v2.role.Role` or None
        """
        return self._find(
            _role.Role, name_or_id, ignore_missing=ignore_missing
        )

    def get_role(self, role):
        """Get a single role

        :param role: The value can be the ID of a role or a
            :class:`~openstack.identity.v2.role.Role` instance.

        :returns: One :class:`~openstack.identity.v2.role.Role`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_role.Role, role)

    def roles(self, **query):
        """Retrieve a generator of roles

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of role instances.
        :rtype: :class:`~openstack.identity.v2.role.Role`
        """
        return self._list(_role.Role, **query)

    def update_role(self, role, **attrs):
        """Update a role

        :param role: Either the ID of a role or a
            :class:`~openstack.identity.v2.role.Role` instance.
        :param attrs: The attributes to update on the role represented
            by ``role``.

        :returns: The updated role
        :rtype: :class:`~openstack.identity.v2.role.Role`
        """
        return self._update(_role.Role, role, **attrs)

    def create_tenant(self, **attrs):
        """Create a new tenant from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.identity.v2.tenant.Tenant`,
            comprised of the properties on the Tenant class.

        :returns: The results of tenant creation
        :rtype: :class:`~openstack.identity.v2.tenant.Tenant`
        """
        return self._create(_tenant.Tenant, **attrs)

    def delete_tenant(self, tenant, ignore_missing=True):
        """Delete a tenant

        :param tenant: The value can be either the ID of a tenant or a
            :class:`~openstack.identity.v2.tenant.Tenant` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the tenant does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent tenant.

        :returns: ``None``
        """
        self._delete(_tenant.Tenant, tenant, ignore_missing=ignore_missing)

    def find_tenant(self, name_or_id, ignore_missing=True):
        """Find a single tenant

        :param name_or_id: The name or ID of a tenant.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.identity.v2.tenant.Tenant` or None
        """
        return self._find(
            _tenant.Tenant, name_or_id, ignore_missing=ignore_missing
        )

    def get_tenant(self, tenant):
        """Get a single tenant

        :param tenant: The value can be the ID of a tenant or a
            :class:`~openstack.identity.v2.tenant.Tenant` instance.

        :returns: One :class:`~openstack.identity.v2.tenant.Tenant`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_tenant.Tenant, tenant)

    def tenants(self, **query):
        """Retrieve a generator of tenants

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of tenant instances.
        :rtype: :class:`~openstack.identity.v2.tenant.Tenant`
        """
        return self._list(_tenant.Tenant, **query)

    def update_tenant(self, tenant, **attrs):
        """Update a tenant

        :param tenant: Either the ID of a tenant or a
            :class:`~openstack.identity.v2.tenant.Tenant` instance.
        :param attrs: The attributes to update on the tenant represented
            by ``tenant``.

        :returns: The updated tenant
        :rtype: :class:`~openstack.identity.v2.tenant.Tenant`
        """
        return self._update(_tenant.Tenant, tenant, **attrs)

    def create_user(self, **attrs):
        """Create a new user from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.identity.v2.user.User`,
            comprised of the properties on the User class.

        :returns: The results of user creation
        :rtype: :class:`~openstack.identity.v2.user.User`
        """
        return self._create(_user.User, **attrs)

    def delete_user(self, user, ignore_missing=True):
        """Delete a user

        :param user: The value can be either the ID of a user or a
            :class:`~openstack.identity.v2.user.User` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the user does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent user.

        :returns: ``None``
        """
        self._delete(_user.User, user, ignore_missing=ignore_missing)

    def find_user(self, name_or_id, ignore_missing=True):
        """Find a single user

        :param name_or_id: The name or ID of a user.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.identity.v2.user.User` or None
        """
        return self._find(
            _user.User, name_or_id, ignore_missing=ignore_missing
        )

    def get_user(self, user):
        """Get a single user

        :param user: The value can be the ID of a user or a
            :class:`~openstack.identity.v2.user.User` instance.

        :returns: One :class:`~openstack.identity.v2.user.User`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_user.User, user)

    def users(self, **query):
        """Retrieve a generator of users

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of user instances.
        :rtype: :class:`~openstack.identity.v2.user.User`
        """
        return self._list(_user.User, **query)

    def update_user(self, user, **attrs):
        """Update a user

        :param user: Either the ID of a user or a
            :class:`~openstack.identity.v2.user.User` instance.
        :param attrs: The attributes to update on the user represented
            by ``user``.

        :returns: The updated user
        :rtype: :class:`~openstack.identity.v2.user.User`
        """
        return self._update(_user.User, user, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import sfc_flow_classifier
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    "description": "",
    "project_id": "4ad57e7ce0b24fca8f12b9834d91079d",
    "tenant_id": "4ad57e7ce0b24fca8f12b9834d91079d",
    "ethertype": "IPv4",
    "protocol": 6,
    "source_port_range_min": 22,
    "source_port_range_max": 2000,
    "destination_port_range_min": 80,
    "destination_port_range_max": 80,
    "source_ip_prefix": None,
    "destination_ip_prefix": "22.12.34.45",
    "logical_source_port": "uuid1",
    "logical_destination_port": "uuid2",
    "l7_parameters": None,
    "id": "6ecd9cf3-ca64-46c7-863f-f2eb1b9e838a",
    "name": "flow_classifier",
}


class TestFlowClassifier(base.TestCase):
    def test_basic(self):
        sot = sfc_flow_classifier.SfcFlowClassifier()
        self.assertEqual('flow_classifier', sot.resource_key)
        self.assertEqual('flow_classifiers', sot.resources_key)
        self.assertEqual('/sfc/flow_classifiers', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = sfc_flow_classifier.SfcFlowClassifier(**EXAMPLE)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['protocol'], sot.protocol)
        self.assertEqual(EXAMPLE['ethertype'], sot.ethertype)
        self.assertEqual(
            EXAMPLE['source_port_range_min'], sot.source_port_range_min
        )
        self.assertEqual(
            EXAMPLE['source_port_range_max'], sot.source_port_range_max
        )
        self.assertEqual(
            EXAMPLE['destination_port_range_min'],
            sot.destination_port_range_min,
        )
        self.assertEqual(
            EXAMPLE['destination_port_range_max'],
            sot.destination_port_range_max,
        )
        self.assertEqual(EXAMPLE['source_ip_prefix'], sot.source_ip_prefix)
        self.assertEqual(
            EXAMPLE['destination_ip_prefix'], sot.destination_ip_prefix
        )
        self.assertEqual(
            EXAMPLE['logical_source_port'], sot.logical_source_port
        )
        self.assertEqual(
            EXAMPLE['logical_destination_port'], sot.logical_destination_port
        )
        self.assertEqual(EXAMPLE['l7_parameters'], sot.l7_parameters)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)

        self.assertDictEqual(
            {
                "limit": "limit",
                "marker": "marker",
                'description': 'description',
                'name': 'name',
                'project_id': 'project_id',
                'tenant_id': 'tenant_id',
                'ethertype': 'ethertype',
                'protocol': 'protocol',
                'source_port_range_min': 'source_port_range_min',
                'source_port_range_max': 'source_port_range_max',
                'destination_port_range_min': 'destination_port_range_min',
                'destination_port_range_max': 'destination_port_range_max',
                'logical_source_port': 'logical_source_port',
                'logical_destination_port': 'logical_destination_port',
            },
            sot._query_mapping._mapping,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import qos_policy as _qos_policy
from openstack.tests.functional import base


class TestQoSPolicy(base.BaseFunctionalTest):
    QOS_POLICY_ID = None
    IS_SHARED = False
    IS_DEFAULT = False
    RULES = []
    QOS_POLICY_DESCRIPTION = "QoS policy description"

    def setUp(self):
        super(TestQoSPolicy, self).setUp()

        if not self.operator_cloud:
            self.skipTest("Operator cloud is required for this test")

        # Skip the tests if qos extension is not enabled.
        if not self.operator_cloud.network.find_extension("qos"):
            self.skipTest("Network qos extension disabled")

        self.QOS_POLICY_NAME = self.getUniqueString()
        self.QOS_POLICY_NAME_UPDATED = self.getUniqueString()
        qos = self.operator_cloud.network.create_qos_policy(
            description=self.QOS_POLICY_DESCRIPTION,
            name=self.QOS_POLICY_NAME,
            shared=self.IS_SHARED,
            is_default=self.IS_DEFAULT,
        )
        assert isinstance(qos, _qos_policy.QoSPolicy)
        self.assertEqual(self.QOS_POLICY_NAME, qos.name)
        self.QOS_POLICY_ID = qos.id

    def tearDown(self):
        sot = self.operator_cloud.network.delete_qos_policy(self.QOS_POLICY_ID)
        self.assertIsNone(sot)
        super(TestQoSPolicy, self).tearDown()

    def test_find(self):
        sot = self.operator_cloud.network.find_qos_policy(self.QOS_POLICY_NAME)
        self.assertEqual(self.QOS_POLICY_ID, sot.id)

    def test_get(self):
        sot = self.operator_cloud.network.get_qos_policy(self.QOS_POLICY_ID)
        self.assertEqual(self.QOS_POLICY_NAME, sot.name)
        self.assertEqual(self.IS_SHARED, sot.is_shared)
        self.assertEqual(self.RULES, sot.rules)
        self.assertEqual(self.QOS_POLICY_DESCRIPTION, sot.description)
        self.assertEqual(self.IS_DEFAULT, sot.is_default)

    def test_list(self):
        names = [o.name for o in self.operator_cloud.network.qos_policies()]
        self.assertIn(self.QOS_POLICY_NAME, names)

    def test_update(self):
        sot = self.operator_cloud.network.update_qos_policy(
            self.QOS_POLICY_ID, name=self.QOS_POLICY_NAME_UPDATED
        )
        self.assertEqual(self.QOS_POLICY_NAME_UPDATED, sot.name)

    def test_set_tags(self):
        sot = self.operator_cloud.network.get_qos_policy(self.QOS_POLICY_ID)
        self.assertEqual([], sot.tags)

        self.operator_cloud.network.set_tags(sot, ["blue"])
        sot = self.operator_cloud.network.get_qos_policy(self.QOS_POLICY_ID)
        self.assertEqual(["blue"], sot.tags)

        self.operator_cloud.network.set_tags(sot, [])
        sot = self.operator_cloud.network.get_qos_policy(self.QOS_POLICY_ID)
        self.assertEqual([], sot.tags)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.functional.shared_file_system import base


class LimitTest(base.BaseSharedFileSystemTest):
    def test_limits(self):
        limits = self.user_cloud.shared_file_system.limits()
        self.assertGreater(len(list(limits)), 0)
        for limit in limits:
            for attribute in (
                "maxTotalReplicaGigabytes",
                "maxTotalShares",
                "maxTotalShareGigabytes",
                "maxTotalShareNetworks",
                "maxTotalShareSnapshots",
                "maxTotalShareReplicas",
                "maxTotalSnapshotGigabytes",
                "totalReplicaGigabytesUsed",
                "totalShareGigabytesUsed",
                "totalSharesUsed",
                "totalShareNetworksUsed",
                "totalShareSnapshotsUsed",
                "totalSnapshotGigabytesUsed",
                "totalShareReplicasUsed",
            ):
                self.assertTrue(hasattr(limit, attribute))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.key_manager.v1 import container as _container
from openstack.key_manager.v1 import order as _order
from openstack.key_manager.v1 import secret as _secret
from openstack import proxy


class Proxy(proxy.Proxy):
    _resource_registry = {
        "container": _container.Container,
        "order": _order.Order,
        "secret": _secret.Secret,
    }

    def create_container(self, **attrs):
        """Create a new container from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.key_manager.v1.container.Container`,
            comprised of the properties on the Container class.

        :returns: The results of container creation
        :rtype: :class:`~openstack.key_manager.v1.container.Container`
        """
        return self._create(_container.Container, **attrs)

    def delete_container(self, container, ignore_missing=True):
        """Delete a container

        :param container: The value can be either the ID of a container or a
            :class:`~openstack.key_manager.v1.container.Container`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the container does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent container.

        :returns: ``None``
        """
        self._delete(
            _container.Container, container, ignore_missing=ignore_missing
        )

    def find_container(self, name_or_id, ignore_missing=True):
        """Find a single container

        :param name_or_id: The name or ID of a container.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.key_manager.v1.container.Container`
            or None
        """
        return self._find(
            _container.Container, name_or_id, ignore_missing=ignore_missing
        )

    def get_container(self, container):
        """Get a single container

        :param container: The value can be the ID of a container or a
            :class:`~openstack.key_manager.v1.container.Container`
            instance.

        :returns: One :class:`~openstack.key_manager.v1.container.Container`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_container.Container, container)

    def containers(self, **query):
        """Return a generator of containers

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of container objects
        :rtype: :class:`~openstack.key_manager.v1.container.Container`
        """
        return self._list(_container.Container, **query)

    def update_container(self, container, **attrs):
        """Update a container

        :param container: Either the id of a container or a
            :class:`~openstack.key_manager.v1.container.Container` instance.
        :param attrs: The attributes to update on the container represented
            by ``container``.

        :returns: The updated container
        :rtype: :class:`~openstack.key_manager.v1.container.Container`
        """
        return self._update(_container.Container, container, **attrs)

    def create_order(self, **attrs):
        """Create a new order from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.key_manager.v1.order.Order`,
            comprised of the properties on the Order class.

        :returns: The results of order creation
        :rtype: :class:`~openstack.key_manager.v1.order.Order`
        """
        return self._create(_order.Order, **attrs)

    def delete_order(self, order, ignore_missing=True):
        """Delete an order

        :param order: The value can be either the ID of a order or a
            :class:`~openstack.key_manager.v1.order.Order`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the order does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent order.

        :returns: ``None``
        """
        self._delete(_order.Order, order, ignore_missing=ignore_missing)

    def find_order(self, name_or_id, ignore_missing=True):
        """Find a single order

        :param name_or_id: The name or ID of a order.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.key_manager.v1.order.Order` or None
        """
        return self._find(
            _order.Order, name_or_id, ignore_missing=ignore_missing
        )

    def get_order(self, order):
        """Get a single order

        :param order: The value can be the ID of an order or a
            :class:`~openstack.key_manager.v1.order.Order`
            instance.

        :returns: One :class:`~openstack.key_manager.v1.order.Order`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_order.Order, order)

    def orders(self, **query):
        """Return a generator of orders

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of order objects
        :rtype: :class:`~openstack.key_manager.v1.order.Order`
        """
        return self._list(_order.Order, **query)

    def update_order(self, order, **attrs):
        """Update a order

        :param order: Either the id of a order or a
            :class:`~openstack.key_manager.v1.order.Order` instance.
        :param attrs: The attributes to update on the order represented
            by ``order``.

        :returns: The updated order
        :rtype: :class:`~openstack.key_manager.v1.order.Order`
        """
        return self._update(_order.Order, order, **attrs)

    def create_secret(self, **attrs):
        """Create a new secret from attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~openstack.key_manager.v1.secret.Secret`,
            comprised of the properties on the Order class.

        :returns: The results of secret creation
        :rtype: :class:`~openstack.key_manager.v1.secret.Secret`
        """
        return self._create(_secret.Secret, **attrs)

    def delete_secret(self, secret, ignore_missing=True):
        """Delete a secret

        :param secret: The value can be either the ID of a secret or a
            :class:`~openstack.key_manager.v1.secret.Secret`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the secret does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent secret.

        :returns: ``None``
        """
        self._delete(_secret.Secret, secret, ignore_missing=ignore_missing)

    def find_secret(self, name_or_id, ignore_missing=True):
        """Find a single secret

        :param name_or_id: The name or ID of a secret.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.key_manager.v1.secret.Secret` or
            None
        """
        return self._find(
            _secret.Secret, name_or_id, ignore_missing=ignore_missing
        )

    def get_secret(self, secret):
        """Get a single secret

        :param secret: The value can be the ID of a secret or a
            :class:`~openstack.key_manager.v1.secret.Secret`
            instance.

        :returns: One :class:`~openstack.key_manager.v1.secret.Secret`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_secret.Secret, secret)

    def secrets(self, **query):
        """Return a generator of secrets

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of secret objects
        :rtype: :class:`~openstack.key_manager.v1.secret.Secret`
        """
        return self._list(_secret.Secret, **query)

    def update_secret(self, secret, **attrs):
        """Update a secret

        :param secret: Either the id of a secret or a
            :class:`~openstack.key_manager.v1.secret.Secret` instance.
        :param attrs: The attributes to update on the secret represented
            by ``secret``.

        :returns: The updated secret
        :rtype: :class:`~openstack.key_manager.v1.secret.Secret`
        """
        return self._update(_secret.Secret, secret, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource


class Trait(resource.Resource):
    resource_key = None
    resources_key = None
    base_path = '/traits'

    # Capabilities

    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    create_method = 'PUT'

    # Added in 1.6
    _max_microversion = '1.6'

    _query_mapping = resource.QueryParameters(
        'name',
        'associated',
        include_pagination_defaults=False,
    )

    name = resource.Body('name', alternate_id=True)

    @classmethod
    def list(
        cls,
        session,
        paginated=True,
        base_path=None,
        allow_unknown_params=False,
        *,
        microversion=None,
        **params,
    ):
        """This method is a generator which yields resource objects.

        A re-implementation of :meth:`~openstack.resource.Resource.list` that
        handles the list of strings (as opposed to a list of objects) that this
        call returns.

        Refer to :meth:`~openstack.resource.Resource.list` for full
        documentation including parameter, exception and return type
        documentation.
        """
        session = cls._get_session(session)

        if microversion is None:
            microversion = cls._get_microversion(session, action='list')

        if base_path is None:
            base_path = cls.base_path

        # There is no server-side filtering, only client-side
        client_filters = {}
        # Gather query parameters which are not supported by the server
        for k, v in params.items():
            if (
                # Known attr
                hasattr(cls, k)
                # Is real attr property
                and isinstance(getattr(cls, k), resource.Body)
                # not included in the query_params
                and k not in cls._query_mapping._mapping.keys()
            ):
                client_filters[k] = v

        uri = base_path % params
        uri_params = {}

        for k, v in params.items():
            # We need to gather URI parts to set them on the resource later
            if hasattr(cls, k) and isinstance(getattr(cls, k), resource.URI):
                uri_params[k] = v

        def _dict_filter(f, d):
            """Dict param based filtering"""
            if not d:
                return False
            for key in f.keys():
                if isinstance(f[key], dict):
                    if not _dict_filter(f[key], d.get(key, None)):
                        return False
                elif d.get(key, None) != f[key]:
                    return False
            return True

        response = session.get(
            uri,
            headers={"Accept": "application/json"},
            params={},
            microversion=microversion,
        )
        exceptions.raise_from_response(response)
        data = response.json()

        for trait_name in data['traits']:
            trait = {
                'name': trait_name,
                **uri_params,
            }
            value = cls.existing(
                microversion=microversion,
                connection=session._get_connection(),
                **trait,
            )

            filters_matched = True
            # Iterate over client filters and return only if matching
            for key in client_filters.keys():
                if isinstance(client_filters[key], dict):
                    if not _dict_filter(
                        client_filters[key],
                        value.get(key, None),
                    ):
                        filters_matched = False
                        break
                elif value.get(key, None) != client_filters[key]:
                    filters_matched = False
                    break

            if filters_matched:
                yield value

        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
from unittest import mock

from keystoneauth1 import exceptions as ksa_exceptions
from keystoneauth1 import session as ksa_session

from openstack.config import cloud_region
from openstack.config import defaults
from openstack import exceptions
from openstack.tests.unit.config import base
from openstack import version as openstack_version

fake_config_dict = {'a': 1, 'os_b': 2, 'c': 3, 'os_c': 4}
fake_services_dict = {
    'compute_api_version': '2',
    'compute_endpoint_override': 'http://compute.example.com',
    'telemetry_endpoint': 'http://telemetry.example.com',
    'interface': 'public',
    'image_service_type': 'mage',
    'identity_interface': 'admin',
    'identity_service_name': 'locks',
    'volume_api_version': '1',
    'auth': {'password': 'hunter2', 'username': 'AzureDiamond'},
    'connect_retries': 1,
    'baremetal_status_code_retries': 5,
    'baremetal_connect_retries': 3,
}


class TestCloudRegion(base.TestCase):
    def test_arbitrary_attributes(self):
        cc = cloud_region.CloudRegion("test1", "region-al", fake_config_dict)
        self.assertEqual("test1", cc.name)
        self.assertEqual("region-al", cc.region_name)

        # Look up straight value
        self.assertEqual("1", cc.a)

        # Look up prefixed attribute, fail - returns None
        self.assertIsNone(cc.os_b)

        # Look up straight value, then prefixed value
        self.assertEqual("3", cc.c)
        self.assertEqual("3", cc.os_c)

        # Lookup mystery attribute
        self.assertIsNone(cc.x)

        # Test default ipv6
        self.assertFalse(cc.force_ipv4)

    def test_iteration(self):
        cc = cloud_region.CloudRegion("test1", "region-al", fake_config_dict)
        self.assertIn('a', cc)
        self.assertNotIn('x', cc)

    def test_equality(self):
        cc1 = cloud_region.CloudRegion("test1", "region-al", fake_config_dict)
        cc2 = cloud_region.CloudRegion("test1", "region-al", fake_config_dict)
        self.assertEqual(cc1, cc2)

    def test_inequality(self):
        cc1 = cloud_region.CloudRegion("test1", "region-al", fake_config_dict)

        cc2 = cloud_region.CloudRegion("test2", "region-al", fake_config_dict)
        self.assertNotEqual(cc1, cc2)

        cc2 = cloud_region.CloudRegion("test1", "region-xx", fake_config_dict)
        self.assertNotEqual(cc1, cc2)

        cc2 = cloud_region.CloudRegion("test1", "region-al", {})
        self.assertNotEqual(cc1, cc2)

    def test_get_config(self):
        cc = cloud_region.CloudRegion("test1", "region-al", fake_services_dict)
        self.assertIsNone(cc._get_config('nothing', None))
        # This is what is happening behind the scenes in get_default_interface.
        self.assertEqual(
            fake_services_dict['interface'], cc._get_config('interface', None)
        )
        # The same call as above, but from one step up the stack
        self.assertEqual(fake_services_dict['interface'], cc.get_interface())
        # Which finally is what is called to populate the below
        self.assertEqual('public', self.cloud.default_interface)

    def test_verify(self):
        config_dict = copy.deepcopy(fake_config_dict)
        config_dict['cacert'] = None

        config_dict['verify'] = False
        cc = cloud_region.CloudRegion("test1", "region-xx", config_dict)
        (verify, cert) = cc.get_requests_verify_args()
        self.assertFalse(verify)

        config_dict['verify'] = True
        cc = cloud_region.CloudRegion("test1", "region-xx", config_dict)
        (verify, cert) = cc.get_requests_verify_args()
        self.assertTrue(verify)

        config_dict['insecure'] = True
        cc = cloud_region.CloudRegion("test1", "region-xx", config_dict)
        (verify, cert) = cc.get_requests_verify_args()
        self.assertFalse(verify)

    def test_verify_cacert(self):
        config_dict = copy.deepcopy(fake_config_dict)
        config_dict['cacert'] = "certfile"

        config_dict['verify'] = False
        cc = cloud_region.CloudRegion("test1", "region-xx", config_dict)
        (verify, cert) = cc.get_requests_verify_args()
        self.assertFalse(verify)

        config_dict['verify'] = True
        cc = cloud_region.CloudRegion("test1", "region-xx", config_dict)
        (verify, cert) = cc.get_requests_verify_args()
        self.assertEqual("certfile", verify)

        config_dict['insecure'] = True
        cc = cloud_region.CloudRegion("test1", "region-xx", config_dict)
        (verify, cert) = cc.get_requests_verify_args()
        self.assertEqual(False, verify)

    def test_cert_with_key(self):
        config_dict = copy.deepcopy(fake_config_dict)
        config_dict['cacert'] = None
        config_dict['verify'] = False

        config_dict['cert'] = 'cert'
        config_dict['key'] = 'key'

        cc = cloud_region.CloudRegion("test1", "region-xx", config_dict)
        (verify, cert) = cc.get_requests_verify_args()
        self.assertEqual(("cert", "key"), cert)

    def test_ipv6(self):
        cc = cloud_region.CloudRegion(
            "test1", "region-al", fake_config_dict, force_ipv4=True
        )
        self.assertTrue(cc.force_ipv4)

    def test_getters(self):
        cc = cloud_region.CloudRegion("test1", "region-al", fake_services_dict)

        self.assertEqual(
            ['compute', 'identity', 'image', 'volume'],
            sorted(cc.get_services()),
        )
        self.assertEqual(
            {'password': 'hunter2', 'username': 'AzureDiamond'},
            cc.get_auth_args(),
        )
        self.assertEqual('public', cc.get_interface())
        self.assertEqual('public', cc.get_interface('compute'))
        self.assertEqual('admin', cc.get_interface('identity'))
        self.assertEqual('region-al', cc.region_name)
        self.assertIsNone(cc.get_api_version('image'))
        self.assertEqual('2', cc.get_api_version('compute'))
        self.assertEqual('mage', cc.get_service_type('image'))
        self.assertEqual('compute', cc.get_service_type('compute'))
        self.assertEqual('1', cc.get_api_version('volume'))
        self.assertEqual('block-storage', cc.get_service_type('volume'))
        self.assertEqual(
            'http://compute.example.com', cc.get_endpoint('compute')
        )
        self.assertIsNone(cc.get_endpoint('image'))
        self.assertIsNone(cc.get_service_name('compute'))
        self.assertEqual('locks', cc.get_service_name('identity'))
        self.assertIsNone(cc.get_status_code_retries('compute'))
        self.assertEqual(5, cc.get_status_code_retries('baremetal'))
        self.assertEqual(1, cc.get_connect_retries('compute'))
        self.assertEqual(3, cc.get_connect_retries('baremetal'))

    def test_rackspace_workaround(self):
        # We're skipping loader here, so we have to expand relevant
        # parts from the rackspace profile. The thing we're testing
        # is that the project_id logic works.
        cc = cloud_region.CloudRegion(
            "test1",
            "DFW",
            {
                'profile': 'rackspace',
                'region_name': 'DFW',
                'auth': {'project_id': '123456'},
                'block_storage_endpoint_override': 'https://example.com/v2/',
            },
        )
        self.assertEqual(
            'https://example.com/v2/123456', cc.get_endpoint('block-storage')
        )

    def test_rackspace_workaround_only_rax(self):
        cc = cloud_region.CloudRegion(
            "test1",
            "DFW",
            {
                'region_name': 'DFW',
                'auth': {'project_id': '123456'},
                'block_storage_endpoint_override': 'https://example.com/v2/',
            },
        )
        self.assertEqual(
            'https://example.com/v2/', cc.get_endpoint('block-storage')
        )

    def test_get_region_name(self):
        def assert_region_name(default, compute):
            self.assertEqual(default, cc.region_name)
            self.assertEqual(default, cc.get_region_name())
            self.assertEqual(default, cc.get_region_name(service_type=None))
            self.assertEqual(
                compute, cc.get_region_name(service_type='compute')
            )
            self.assertEqual(
                default, cc.get_region_name(service_type='placement')
            )

        # No region_name kwarg, no regions specified in services dict
        # (including the default).
        cc = cloud_region.CloudRegion(config=fake_services_dict)
        assert_region_name(None, None)

        # Only region_name kwarg; it's returned for everything
        cc = cloud_region.CloudRegion(
            region_name='foo', config=fake_services_dict
        )
        assert_region_name('foo', 'foo')

        # No region_name kwarg; values (including default) show through from
        # config dict
        services_dict = dict(
            fake_services_dict,
            region_name='the-default',
            compute_region_name='compute-region',
        )
        cc = cloud_region.CloudRegion(config=services_dict)
        assert_region_name('the-default', 'compute-region')

        # region_name kwarg overrides config dict default (for backward
        # compatibility), but service-specific region_name takes precedence.
        services_dict = dict(
            fake_services_dict,
            region_name='dict',
            compute_region_name='compute-region',
        )
        cc = cloud_region.CloudRegion(
            region_name='kwarg', config=services_dict
        )
        assert_region_name('kwarg', 'compute-region')

    def test_aliases(self):
        services_dict = fake_services_dict.copy()
        services_dict['volume_api_version'] = 12
        services_dict['alarming_service_name'] = 'aodh'
        cc = cloud_region.CloudRegion("test1", "region-al", services_dict)
        self.assertEqual('12', cc.get_api_version('volume'))
        self.assertEqual('12', cc.get_api_version('block-storage'))
        self.assertEqual('aodh', cc.get_service_name('alarm'))
        self.assertEqual('aodh', cc.get_service_name('alarming'))

    def test_no_override(self):
        """Test no override happens when defaults are not configured"""
        cc = cloud_region.CloudRegion("test1", "region-al", fake_services_dict)
        self.assertEqual('block-storage', cc.get_service_type('volume'))
        self.assertEqual('workflow', cc.get_service_type('workflow'))
        self.assertEqual('not-exist', cc.get_service_type('not-exist'))

    def test_get_session_no_auth(self):
        config_dict = defaults.get_defaults()
        config_dict.update(fake_services_dict)
        cc = cloud_region.CloudRegion("test1", "region-al", config_dict)
        self.assertRaises(exceptions.ConfigException, cc.get_session)

    @mock.patch.object(ksa_session, 'Session')
    def test_get_session(self, mock_session):
        config_dict = defaults.get_defaults()
        config_dict.update(fake_services_dict)
        fake_session = mock.Mock()
        fake_session.additional_user_agent = []
        mock_session.return_value = fake_session
        cc = cloud_region.CloudRegion(
            "test1", "region-al", config_dict, auth_plugin=mock.Mock()
        )
        cc.get_session()
        mock_session.assert_called_with(
            auth=mock.ANY,
            verify=True,
            cert=None,
            timeout=None,
            collect_timing=None,
            discovery_cache=None,
        )
        self.assertEqual(
            fake_session.additional_user_agent,
            [('openstacksdk', openstack_version.__version__)],
        )

    @mock.patch.object(ksa_session, 'Session')
    def test_get_session_with_app_name(self, mock_session):
        config_dict = defaults.get_defaults()
        config_dict.update(fake_services_dict)
        fake_session = mock.Mock()
        fake_session.additional_user_agent = []
        fake_session.app_name = None
        fake_session.app_version = None
        mock_session.return_value = fake_session
        cc = cloud_region.CloudRegion(
            "test1",
            "region-al",
            config_dict,
            auth_plugin=mock.Mock(),
            app_name="test_app",
            app_version="test_version",
        )
        cc.get_session()
        mock_session.assert_called_with(
            auth=mock.ANY,
            verify=True,
            cert=None,
            timeout=None,
            collect_timing=None,
            discovery_cache=None,
        )
        self.assertEqual(fake_session.app_name, "test_app")
        self.assertEqual(fake_session.app_version, "test_version")
        self.assertEqual(
            fake_session.additional_user_agent,
            [('openstacksdk', openstack_version.__version__)],
        )

    @mock.patch.object(ksa_session, 'Session')
    def test_get_session_with_timeout(self, mock_session):
        fake_session = mock.Mock()
        fake_session.additional_user_agent = []
        mock_session.return_value = fake_session
        config_dict = defaults.get_defaults()
        config_dict.update(fake_services_dict)
        config_dict['api_timeout'] = 9
        cc = cloud_region.CloudRegion(
            "test1", "region-al", config_dict, auth_plugin=mock.Mock()
        )
        cc.get_session()
        mock_session.assert_called_with(
            auth=mock.ANY,
            verify=True,
            cert=None,
            timeout=9,
            collect_timing=None,
            discovery_cache=None,
        )
        self.assertEqual(
            fake_session.additional_user_agent,
            [('openstacksdk', openstack_version.__version__)],
        )

    @mock.patch.object(ksa_session, 'Session')
    def test_get_session_with_timing(self, mock_session):
        fake_session = mock.Mock()
        fake_session.additional_user_agent = []
        mock_session.return_value = fake_session
        config_dict = defaults.get_defaults()
        config_dict.update(fake_services_dict)
        config_dict['timing'] = True
        cc = cloud_region.CloudRegion(
            "test1", "region-al", config_dict, auth_plugin=mock.Mock()
        )
        cc.get_session()
        mock_session.assert_called_with(
            auth=mock.ANY,
            verify=True,
            cert=None,
            timeout=None,
            collect_timing=True,
            discovery_cache=None,
        )
        self.assertEqual(
            fake_session.additional_user_agent,
            [('openstacksdk', openstack_version.__version__)],
        )

    @mock.patch.object(ksa_session, 'Session')
    def test_override_session_endpoint_override(self, mock_session):
        config_dict = defaults.get_defaults()
        config_dict.update(fake_services_dict)
        cc = cloud_region.CloudRegion(
            "test1", "region-al", config_dict, auth_plugin=mock.Mock()
        )
        self.assertEqual(
            cc.get_session_endpoint('compute'),
            fake_services_dict['compute_endpoint_override'],
        )

    @mock.patch.object(ksa_session, 'Session')
    def test_override_session_endpoint(self, mock_session):
        config_dict = defaults.get_defaults()
        config_dict.update(fake_services_dict)
        cc = cloud_region.CloudRegion(
            "test1", "region-al", config_dict, auth_plugin=mock.Mock()
        )
        self.assertEqual(
            cc.get_session_endpoint('telemetry'),
            fake_services_dict['telemetry_endpoint'],
        )

    @mock.patch.object(cloud_region.CloudRegion, 'get_session')
    def test_session_endpoint(self, mock_get_session):
        mock_session = mock.Mock()
        mock_get_session.return_value = mock_session
        config_dict = defaults.get_defaults()
        config_dict.update(fake_services_dict)
        cc = cloud_region.CloudRegion(
            "test1", "region-al", config_dict, auth_plugin=mock.Mock()
        )
        cc.get_session_endpoint('orchestration')
        mock_session.get_endpoint.assert_called_with(
            interface='public',
            service_name=None,
            region_name='region-al',
            service_type='orchestration',
        )

    @mock.patch.object(cloud_region.CloudRegion, 'get_session')
    def test_session_endpoint_not_found(self, mock_get_session):
        exc_to_raise = ksa_exceptions.catalog.EndpointNotFound
        mock_get_session.return_value.get_endpoint.side_effect = exc_to_raise
        cc = cloud_region.CloudRegion(
            "test1", "region-al", {}, auth_plugin=mock.Mock()
        )
        self.assertIsNone(cc.get_session_endpoint('notfound'))

    def test_get_endpoint_from_catalog(self):
        dns_override = 'https://override.dns.example.com'
        self.cloud.config.config['dns_endpoint_override'] = dns_override
        self.assertEqual(
            'https://compute.example.com/v2.1/',
            self.cloud.config.get_endpoint_from_catalog('compute'),
        )
        self.assertEqual(
            'https://internal.compute.example.com/v2.1/',
            self.cloud.config.get_endpoint_from_catalog(
                'compute', interface='internal'
            ),
        )
        self.assertIsNone(
            self.cloud.config.get_endpoint_from_catalog(
                'compute', region_name='unknown-region'
            )
        )
        self.assertEqual(
            'https://dns.example.com',
            self.cloud.config.get_endpoint_from_catalog('dns'),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
List resources from the Network service.

For a full guide see
https://docs.openstack.org/openstacksdk/latest/user/guides/network.html
"""


def list_networks(conn):
    print("List Networks:")

    for network in conn.network.networks():
        print(network)


def list_subnets(conn):
    print("List Subnets:")

    for subnet in conn.network.subnets():
        print(subnet)


def list_ports(conn):
    print("List Ports:")

    for port in conn.network.ports():
        print(port)


def list_security_groups(conn):
    print("List Security Groups:")

    for port in conn.network.security_groups():
        print(port)


def list_routers(conn):
    print("List Routers:")

    for router in conn.network.routers():
        print(router)


def list_network_agents(conn):
    print("List Network Agents:")

    for agent in conn.network.agents():
        print(agent)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class SoftwareDeployment(resource.Resource):
    resource_key = 'software_deployment'
    resources_key = 'software_deployments'
    base_path = '/software_deployments'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    # Properties
    #: The stack action that triggers this deployment resource.
    action = resource.Body('action')
    #: The UUID of the software config resource that runs when applying to the
    #: server.
    config_id = resource.Body('config_id')
    #: A map containing the names and values of all inputs to the config.
    input_values = resource.Body('input_values', type=dict)
    #: A map containing the names and values from the deployment.
    output_values = resource.Body('output_values', type=dict)
    #: The UUID of the compute server to which the configuration applies.
    server_id = resource.Body('server_id')
    #: The ID of the authentication project which can also perform operations
    #: on this deployment.
    stack_user_project_id = resource.Body('stack_user_project_id')
    #: Current status of the software deployment.
    status = resource.Body('status')
    #: Error description for the last status change.
    status_reason = resource.Body('status_reason')
    #: The date and time when the software deployment resource was created.
    created_at = resource.Body('creation_time')
    #: The date and time when the software deployment resource was created.
    updated_at = resource.Body('updated_time')

    def create(self, session, base_path=None):
        # This overrides the default behavior of resource creation because
        # heat doesn't accept resource_key in its request.
        return super(SoftwareDeployment, self).create(
            session, prepend_key=False, base_path=base_path
        )

    def commit(self, session, base_path=None):
        # This overrides the default behavior of resource creation because
        # heat doesn't accept resource_key in its request.
        return super(SoftwareDeployment, self).commit(
            session, prepend_key=False, base_path=base_path
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource
from openstack import utils


class TagMixin:
    id: resource.Body
    base_path: str
    _body: resource._ComponentManager

    @classmethod
    def _get_session(cls, session):
        ...

    _tag_query_parameters = {
        'tags': 'tags',
        'any_tags': 'tags-any',
        'not_tags': 'not-tags',
        'not_any_tags': 'not-tags-any',
    }

    #: A list of associated tags
    #: *Type: list of tag strings*
    tags = resource.Body('tags', type=list, default=[])

    def fetch_tags(self, session):
        """Lists tags set on the entity.

        :param session: The session to use for making this request.
        :return: The list with tags attached to the entity
        """
        url = utils.urljoin(self.base_path, self.id, 'tags')
        session = self._get_session(session)
        response = session.get(url)
        exceptions.raise_from_response(response)
        # NOTE(gtema): since this is a common method
        # we can't rely on the resource_key, because tags are returned
        # without resource_key. Do parse response here
        json = response.json()
        if 'tags' in json:
            self._body.attributes.update({'tags': json['tags']})
        return self

    def set_tags(self, session, tags=[]):
        """Sets/Replaces all tags on the resource.

        :param session: The session to use for making this request.
        :param list tags: List with tags to be set on the resource
        """
        url = utils.urljoin(self.base_path, self.id, 'tags')
        session = self._get_session(session)
        response = session.put(url, json={'tags': tags})
        exceptions.raise_from_response(response)
        self._body.attributes.update({'tags': tags})
        return self

    def remove_all_tags(self, session):
        """Removes all tags on the entity.

        :param session: The session to use for making this request.
        """
        url = utils.urljoin(self.base_path, self.id, 'tags')
        session = self._get_session(session)
        response = session.delete(url)
        exceptions.raise_from_response(response)
        self._body.attributes.update({'tags': []})
        return self

    def check_tag(self, session, tag):
        """Checks if tag exists on the entity.

        If the tag does not exist a 404 will be returned

        :param session: The session to use for making this request.
        :param tag: The tag as a string.
        """
        url = utils.urljoin(self.base_path, self.id, 'tags', tag)
        session = self._get_session(session)
        response = session.get(url)
        exceptions.raise_from_response(
            response, error_message='Tag does not exist'
        )
        return self

    def add_tag(self, session, tag):
        """Adds a single tag to the resource.

        :param session: The session to use for making this request.
        :param tag: The tag as a string.
        """
        url = utils.urljoin(self.base_path, self.id, 'tags', tag)
        session = self._get_session(session)
        response = session.put(url)
        exceptions.raise_from_response(response)
        # we do not want to update tags directly
        tags = self.tags
        tags.append(tag)
        self._body.attributes.update({'tags': tags})
        return self

    def remove_tag(self, session, tag):
        """Removes a single tag from the specified resource.

        :param session: The session to use for making this request.
        :param tag: The tag as a string.
        """
        url = utils.urljoin(self.base_path, self.id, 'tags', tag)
        session = self._get_session(session)
        response = session.delete(url)
        exceptions.raise_from_response(response)
        # we do not want to update tags directly
        tags = self.tags
        try:
            # NOTE(gtema): if tags were not fetched, but request suceeded
            # it is ok. Just ensure tag does not exist locally
            tags.remove(tag)
        except ValueError:
            pass  # do nothing!
        self._body.attributes.update({'tags': tags})
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
from unittest import mock

from keystoneauth1 import adapter

from openstack.compute.v2 import hypervisor
from openstack import exceptions
from openstack.tests.unit import base


EXAMPLE = {
    "cpu_info": {
        "arch": "x86_64",
        "model": "Nehalem",
        "vendor": "Intel",
        "features": ["pge", "clflush"],
        "topology": {"cores": 1, "threads": 1, "sockets": 4},
    },
    "state": "up",
    "status": "enabled",
    "servers": [
        {
            "name": "test_server1",
            "uuid": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        },
        {
            "name": "test_server2",
            "uuid": "bbbbbbbb-bbbb-bbbb-bbbb-bbbbbbbbbbbb",
        },
    ],
    "host_ip": "1.1.1.1",
    "hypervisor_hostname": "fake-mini",
    "hypervisor_type": "fake",
    "hypervisor_version": 1000,
    "id": "b1e43b5f-eec1-44e0-9f10-7b4945c0226d",
    "uptime": (
        " 08:32:11 up 93 days, 18:25, 12 users,  "
        "load average: 0.20, 0.12, 0.14"
    ),
    "service": {
        "host": "043b3cacf6f34c90a7245151fc8ebcda",
        "id": "5d343e1d-938e-4284-b98b-6a2b5406ba76",
        "disabled_reason": None,
    },
    # deprecated attributes
    "vcpus_used": 0,
    "local_gb_used": 0,
    "vcpus": 8,
    "memory_mb_used": 512,
    "memory_mb": 7980,
    "current_workload": 0,
    "running_vms": 0,
    "free_disk_gb": 157,
    "disk_available_least": 140,
    "local_gb": 157,
    "free_ram_mb": 7468,
}


class TestHypervisor(base.TestCase):
    def setUp(self):
        super(TestHypervisor, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = 1
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

    def test_basic(self):
        sot = hypervisor.Hypervisor()
        self.assertEqual('hypervisor', sot.resource_key)
        self.assertEqual('hypervisors', sot.resources_key)
        self.assertEqual('/os-hypervisors', sot.base_path)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual(
            {
                'hypervisor_hostname_pattern': 'hypervisor_hostname_pattern',
                'limit': 'limit',
                'marker': 'marker',
                'with_servers': 'with_servers',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = hypervisor.Hypervisor(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['cpu_info'], sot.cpu_info)
        self.assertEqual(EXAMPLE['host_ip'], sot.host_ip)
        self.assertEqual(EXAMPLE['hypervisor_type'], sot.hypervisor_type)
        self.assertEqual(EXAMPLE['hypervisor_version'], sot.hypervisor_version)
        self.assertEqual(EXAMPLE['hypervisor_hostname'], sot.name)
        self.assertEqual(EXAMPLE['service'], sot.service_details)
        self.assertEqual(EXAMPLE['servers'], sot.servers)
        self.assertEqual(EXAMPLE['state'], sot.state)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['uptime'], sot.uptime)
        # Verify deprecated attributes
        self.assertEqual(EXAMPLE['vcpus_used'], sot.vcpus_used)
        self.assertEqual(EXAMPLE['hypervisor_type'], sot.hypervisor_type)
        self.assertEqual(EXAMPLE['local_gb_used'], sot.local_disk_used)
        self.assertEqual(EXAMPLE['vcpus'], sot.vcpus)
        self.assertEqual(EXAMPLE['vcpus_used'], sot.vcpus_used)
        self.assertEqual(EXAMPLE['memory_mb_used'], sot.memory_used)
        self.assertEqual(EXAMPLE['memory_mb'], sot.memory_size)
        self.assertEqual(EXAMPLE['current_workload'], sot.current_workload)
        self.assertEqual(EXAMPLE['running_vms'], sot.running_vms)
        self.assertEqual(EXAMPLE['free_disk_gb'], sot.local_disk_free)
        self.assertEqual(EXAMPLE['disk_available_least'], sot.disk_available)
        self.assertEqual(EXAMPLE['local_gb'], sot.local_disk_size)
        self.assertEqual(EXAMPLE['free_ram_mb'], sot.memory_free)

    @mock.patch(
        'openstack.utils.supports_microversion',
        autospec=True,
        return_value=False,
    )
    def test_get_uptime(self, mv_mock):
        sot = hypervisor.Hypervisor(**copy.deepcopy(EXAMPLE))
        rsp = {
            "hypervisor": {
                "hypervisor_hostname": "fake-mini",
                "id": sot.id,
                "state": "up",
                "status": "enabled",
                "uptime": "08:32:11 up 93 days, 18:25, 12 users",
            }
        }
        resp = mock.Mock()
        resp.body = copy.deepcopy(rsp)
        resp.json = mock.Mock(return_value=resp.body)
        resp.headers = {}
        resp.status_code = 200
        self.sess.get = mock.Mock(return_value=resp)

        hyp = sot.get_uptime(self.sess)
        self.sess.get.assert_called_with(
            'os-hypervisors/{id}/uptime'.format(id=sot.id),
            microversion=self.sess.default_microversion,
        )
        self.assertEqual(rsp['hypervisor']['uptime'], hyp.uptime)
        self.assertEqual(rsp['hypervisor']['status'], sot.status)

    @mock.patch(
        'openstack.utils.supports_microversion',
        autospec=True,
        return_value=True,
    )
    def test_get_uptime_after_2_88(self, mv_mock):
        sot = hypervisor.Hypervisor(**copy.deepcopy(EXAMPLE))
        self.assertRaises(exceptions.SDKException, sot.get_uptime, self.sess)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.load_balancer.v2 import l7_rule
from openstack.tests.unit import base


EXAMPLE = {
    'admin_state_up': True,
    'compare_type': 'REGEX',
    'created_at': '2017-08-17T12:14:57.233772',
    'id': uuid.uuid4(),
    'invert': False,
    'key': 'my_cookie',
    'l7_policy_id': uuid.uuid4(),
    'operating_status': 'ONLINE',
    'project_id': uuid.uuid4(),
    'provisioning_status': 'ACTIVE',
    'type': 'COOKIE',
    'updated_at': '2017-08-17T12:16:57.233772',
    'value': 'chocolate',
}


class TestL7Rule(base.TestCase):
    def test_basic(self):
        test_l7rule = l7_rule.L7Rule()
        self.assertEqual('rule', test_l7rule.resource_key)
        self.assertEqual('rules', test_l7rule.resources_key)
        self.assertEqual(
            '/lbaas/l7policies/%(l7policy_id)s/rules', test_l7rule.base_path
        )
        self.assertTrue(test_l7rule.allow_create)
        self.assertTrue(test_l7rule.allow_fetch)
        self.assertTrue(test_l7rule.allow_commit)
        self.assertTrue(test_l7rule.allow_delete)
        self.assertTrue(test_l7rule.allow_list)

    def test_make_it(self):
        test_l7rule = l7_rule.L7Rule(**EXAMPLE)
        self.assertTrue(test_l7rule.is_admin_state_up)
        self.assertEqual(EXAMPLE['compare_type'], test_l7rule.compare_type)
        self.assertEqual(EXAMPLE['created_at'], test_l7rule.created_at)
        self.assertEqual(EXAMPLE['id'], test_l7rule.id)
        self.assertEqual(EXAMPLE['invert'], test_l7rule.invert)
        self.assertEqual(EXAMPLE['key'], test_l7rule.key)
        self.assertEqual(EXAMPLE['l7_policy_id'], test_l7rule.l7_policy_id)
        self.assertEqual(
            EXAMPLE['operating_status'], test_l7rule.operating_status
        )
        self.assertEqual(EXAMPLE['project_id'], test_l7rule.project_id)
        self.assertEqual(
            EXAMPLE['provisioning_status'], test_l7rule.provisioning_status
        )
        self.assertEqual(EXAMPLE['type'], test_l7rule.type)
        self.assertEqual(EXAMPLE['updated_at'], test_l7rule.updated_at)
        self.assertEqual(EXAMPLE['value'], test_l7rule.rule_value)

        self.assertDictEqual(
            {
                'limit': 'limit',
                'marker': 'marker',
                'created_at': 'created_at',
                'updated_at': 'updated_at',
                'project_id': 'project_id',
                'tags': 'tags',
                'any_tags': 'tags-any',
                'not_tags': 'not-tags',
                'not_any_tags': 'not-tags-any',
                'operating_status': 'operating_status',
                'provisioning_status': 'provisioning_status',
                'is_admin_state_up': 'admin_state_up',
                'compare_type': 'compare_type',
                'invert': 'invert',
                'key': 'key',
                'type': 'type',
                'rule_value': 'rule_value',
                'l7_policy_id': 'l7policy_id',
            },
            test_l7rule._query_mapping._mapping,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import warnings

import testtools

from openstack import exceptions
from openstack import proxy
from openstack.tests.unit import base


class TestMissingVersion(base.TestCase):
    def setUp(self):
        super(TestMissingVersion, self).setUp()
        self.os_fixture.clear_tokens()
        svc = self.os_fixture.v3_token.add_service('image')
        svc.add_endpoint(
            url='https://example.com/image/',
            region='RegionOne',
            interface='public',
        )
        self.use_keystone_v3()
        self.use_glance(
            image_version_json='bad-glance-version.json',
            image_discovery_url='https://example.com/image/',
        )

    def test_unsupported_version(self):
        with testtools.ExpectedException(exceptions.NotSupported):
            self.cloud.image.get('/')

        self.assert_calls()

    def test_unsupported_version_override(self):
        self.cloud.config.config['image_api_version'] = '7'
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            self.assertIsInstance(self.cloud.image, proxy.Proxy)
            self.assertEqual(1, len(w))
            self.assertIn(
                "Service image has no discoverable version.",
                str(w[-1].message),
            )
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class HealthMonitor(resource.Resource):
    resource_key = 'healthmonitor'
    resources_key = 'healthmonitors'
    base_path = '/lbaas/healthmonitors'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'delay',
        'expected_codes',
        'http_method',
        'max_retries',
        'timeout',
        'type',
        'url_path',
        'project_id',
        is_admin_state_up='adminstate_up',
    )

    # Properties
    #: The time, in seconds, between sending probes to members.
    delay = resource.Body('delay')
    #: Expected HTTP codes for a passing HTTP(S) monitor.
    expected_codes = resource.Body('expected_codes')
    #: The HTTP method that the monitor uses for requests.
    http_method = resource.Body('http_method')
    #: The administrative state of the health monitor, which is up
    #: ``True`` or down ``False``. *Type: bool*
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Maximum consecutive health probe tries.
    max_retries = resource.Body('max_retries')
    #: Name of the health monitor.
    name = resource.Body('name')
    #: List of pools associated with this health monitor
    #: *Type: list of dicts which contain the pool IDs*
    pool_ids = resource.Body('pools', type=list)
    #: The ID of the pool associated with this health monitor
    pool_id = resource.Body('pool_id')
    #: The ID of the project this health monitor is associated with.
    project_id = resource.Body('project_id', alias='tenant_id')
    #: Tenant_id (deprecated attribute).
    tenant_id = resource.Body('tenant_id', deprecated=True)
    #: The maximum number of seconds for a monitor to wait for a
    #: connection to be established before it times out. This value must
    #: be less than the delay value.
    timeout = resource.Body('timeout')
    #: The type of probe sent by the load balancer to verify the member
    #: state, which is PING, TCP, HTTP, or HTTPS.
    type = resource.Body('type')
    #: Path portion of URI that will be probed if type is HTTP(S).
    url_path = resource.Body('url_path')



# Copyright 2017 OVH SAS
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack import exceptions
from openstack.network.v2 import qos_rule_type
from openstack.tests.unit import base


class TestQosRuleType(base.TestCase):
    rule_type_name = "bandwidth_limit"

    qos_extension = {
        "updated": "2015-06-08T10:00:00-00:00",
        "name": "Quality of Service",
        "links": [],
        "alias": "qos",
        "description": "The Quality of Service extension.",
    }
    qos_rule_type_details_extension = {
        "updated": "2017-06-22T10:00:00-00:00",
        "name": "Details of QoS rule types",
        "links": [],
        "alias": "qos-rule-type-details",
        "description": (
            "Expose details about QoS rule types supported by "
            "loaded backend drivers"
        ),
    }

    mock_rule_type_bandwidth_limit = {'type': 'bandwidth_limit'}
    mock_rule_type_dscp_marking = {'type': 'dscp_marking'}
    mock_rule_types = [
        mock_rule_type_bandwidth_limit,
        mock_rule_type_dscp_marking,
    ]

    mock_rule_type_details = {
        'drivers': [
            {
                'name': 'linuxbridge',
                'supported_parameters': [
                    {
                        'parameter_values': {'start': 0, 'end': 2147483647},
                        'parameter_type': 'range',
                        'parameter_name': u'max_kbps',
                    },
                    {
                        'parameter_values': ['ingress', 'egress'],
                        'parameter_type': 'choices',
                        'parameter_name': u'direction',
                    },
                    {
                        'parameter_values': {'start': 0, 'end': 2147483647},
                        'parameter_type': 'range',
                        'parameter_name': 'max_burst_kbps',
                    },
                ],
            }
        ],
        'type': rule_type_name,
    }

    def _compare_rule_types(self, exp, real):
        self.assertDictEqual(
            qos_rule_type.QoSRuleType(**exp).to_dict(computed=False),
            real.to_dict(computed=False),
        )

    def test_list_qos_rule_types(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'extensions']
                    ),
                    json={'extensions': [self.qos_extension]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'qos', 'rule-types'],
                    ),
                    json={'rule_types': self.mock_rule_types},
                ),
            ]
        )
        rule_types = self.cloud.list_qos_rule_types()
        for a, b in zip(self.mock_rule_types, rule_types):
            self._compare_rule_types(a, b)
        self.assert_calls()

    def test_list_qos_rule_types_no_qos_extension(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'extensions']
                    ),
                    json={'extensions': []},
                )
            ]
        )
        self.assertRaises(
            exceptions.SDKException, self.cloud.list_qos_rule_types
        )
        self.assert_calls()

    def test_get_qos_rule_type_details(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'extensions']
                    ),
                    json={
                        'extensions': [
                            self.qos_extension,
                            self.qos_rule_type_details_extension,
                        ]
                    },
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'extensions']
                    ),
                    json={
                        'extensions': [
                            self.qos_extension,
                            self.qos_rule_type_details_extension,
                        ]
                    },
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=[
                            'v2.0',
                            'qos',
                            'rule-types',
                            self.rule_type_name,
                        ],
                    ),
                    json={'rule_type': self.mock_rule_type_details},
                ),
            ]
        )

        self._compare_rule_types(
            self.mock_rule_type_details,
            self.cloud.get_qos_rule_type_details(self.rule_type_name),
        )
        self.assert_calls()

    def test_get_qos_rule_type_details_no_qos_extension(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'extensions']
                    ),
                    json={'extensions': []},
                )
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.get_qos_rule_type_details,
            self.rule_type_name,
        )
        self.assert_calls()

    def test_get_qos_rule_type_details_no_qos_details_extension(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'extensions']
                    ),
                    json={'extensions': [self.qos_extension]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'extensions']
                    ),
                    json={'extensions': [self.qos_extension]},
                ),
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.get_qos_rule_type_details,
            self.rule_type_name,
        )
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import random

from openstack import connection
from openstack import exceptions
from openstack.tests.functional import base
from openstack import utils


class TestZone(base.BaseFunctionalTest):
    def setUp(self):
        super(TestZone, self).setUp()
        self.require_service('dns')

        self.conn = connection.from_config(cloud_name=base.TEST_CLOUD_NAME)

        # Note: zone deletion is not an immediate operation, so each time
        # chose a new zone name for a test
        # getUniqueString is not guaranteed to return unique string between
        # different tests of the same class.
        self.ZONE_NAME = 'example-{0}.org.'.format(random.randint(1, 10000))

        self.zone = self.conn.dns.create_zone(
            name=self.ZONE_NAME,
            email='joe@example.org',
            type='PRIMARY',
            ttl=7200,
            description='example zone',
        )
        self.addCleanup(self.conn.dns.delete_zone, self.zone)

    def test_get_zone(self):
        zone = self.conn.dns.get_zone(self.zone)
        self.assertEqual(self.zone, zone)

    def test_list_zones(self):
        names = [f.name for f in self.conn.dns.zones()]
        self.assertIn(self.ZONE_NAME, names)

    def test_update_zone(self):
        current_ttl = self.conn.dns.get_zone(self.zone)['ttl']
        self.conn.dns.update_zone(self.zone, ttl=current_ttl + 1)
        updated_zone_ttl = self.conn.dns.get_zone(self.zone)['ttl']
        self.assertEqual(
            current_ttl + 1,
            updated_zone_ttl,
            'Failed, updated TTL value is:{} instead of expected:{}'.format(
                updated_zone_ttl, current_ttl + 1
            ),
        )

    def test_create_rs(self):
        zone = self.conn.dns.get_zone(self.zone)
        self.assertIsNotNone(
            self.conn.dns.create_recordset(
                zone=zone,
                name='www.{zone}'.format(zone=zone.name),
                type='A',
                description='Example zone rec',
                ttl=3600,
                records=['192.168.1.1'],
            )
        )

    def test_delete_zone_with_shares(self):
        # Make sure the API under test has shared zones support
        if not utils.supports_version(self.conn.dns, '2.1'):
            self.skipTest(
                'Designate API version does not support shared zones.'
            )

        zone_name = 'example-{0}.org.'.format(random.randint(1, 10000))
        zone = self.conn.dns.create_zone(
            name=zone_name,
            email='joe@example.org',
            type='PRIMARY',
            ttl=7200,
            description='example zone',
        )
        self.addCleanup(self.conn.dns.delete_zone, zone)

        demo_project_id = self.operator_cloud.get_project('demo')['id']
        zone_share = self.conn.dns.create_zone_share(
            zone, target_project_id=demo_project_id
        )
        self.addCleanup(self.conn.dns.delete_zone_share, zone, zone_share)

        # Test that we cannot delete a zone with shares
        self.assertRaises(
            exceptions.BadRequestException, self.conn.dns.delete_zone, zone
        )

        self.conn.dns.delete_zone(zone, delete_shares=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack import resource


class Queue(resource.Resource):
    # FIXME(anyone): The name string of `location` field of Zaqar API response
    # is lower case. That is inconsistent with the guide from API-WG. This is
    # a workaround for this issue.
    location = resource.Header("location")

    resources_key = "queues"
    base_path = "/queues"

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True

    # Properties
    #: The default TTL of messages defined for a queue, which will effect for
    #: any messages posted to the queue.
    default_message_ttl = resource.Body("_default_message_ttl")
    #: Description of the queue.
    description = resource.Body("description")
    #: The max post size of messages defined for a queue, which will effect
    #: for any messages posted to the queue.
    max_messages_post_size = resource.Body("_max_messages_post_size")
    #: Name of the queue. The name is the unique identity of a queue. It
    #: must not exceed 64 bytes in length, and it is limited to US-ASCII
    #: letters, digits, underscores, and hyphens.
    name = resource.Body("name", alternate_id=True)
    #: The ID to identify the client accessing Zaqar API. Must be specified
    #: in header for each API request.
    client_id = resource.Header("Client-ID")
    #: The ID to identify the project accessing Zaqar API. Must be specified
    #: in case keystone auth is not enabled in Zaqar service.
    project_id = resource.Header("X-PROJECT-ID")

    def create(self, session, prepend_key=True, base_path=None):
        request = self._prepare_request(
            requires_id=True, prepend_key=prepend_key, base_path=None
        )
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id(),
        }
        request.headers.update(headers)
        response = session.put(
            request.url, json=request.body, headers=request.headers
        )

        self._translate_response(response, has_body=False)
        return self

    @classmethod
    def list(cls, session, paginated=False, base_path=None, **params):
        """This method is a generator which yields queue objects.

        This is almost the copy of list method of resource.Resource class.
        The only difference is the request header now includes `Client-ID`
        and `X-PROJECT-ID` fields which are required by Zaqar v2 API.
        """
        more_data = True
        query_params = cls._query_mapping._transpose(params, cls)

        if base_path is None:
            base_path = cls.base_path

        uri = base_path % params
        headers = {
            "Client-ID": params.get('client_id', None) or str(uuid.uuid4()),
            "X-PROJECT-ID": params.get('project_id', None)
            or session.get_project_id(),
        }

        while more_data:
            resp = session.get(uri, headers=headers, params=query_params)
            resp = resp.json()
            resp = resp[cls.resources_key]

            if not resp:
                more_data = False

            yielded = 0
            new_marker = None
            for data in resp:
                value = cls.existing(**data)
                new_marker = value.id
                yielded += 1
                yield value

            if not paginated:
                return
            if "limit" in query_params and yielded < query_params["limit"]:
                return
            query_params["limit"] = yielded
            query_params["marker"] = new_marker

    def fetch(
        self,
        session,
        requires_id=True,
        base_path=None,
        error_message=None,
        skip_cache=False,
    ):
        request = self._prepare_request(
            requires_id=requires_id, base_path=base_path
        )
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id(),
        }
        request.headers.update(headers)
        response = session.get(
            request.url, headers=headers, skip_cache=skip_cache
        )
        self._translate_response(response)

        return self

    def delete(self, session):
        request = self._prepare_request()
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id(),
        }
        request.headers.update(headers)
        response = session.delete(request.url, headers=headers)

        self._translate_response(response, has_body=False)
        return self



# Copyright (c) 2018 China Telecom Corporation
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import firewall_rule
from openstack.tests.functional import base


class TestFirewallRule(base.BaseFunctionalTest):
    ACTION = "allow"
    DEST_IP = "10.0.0.0/24"
    DEST_PORT = "80"
    IP_VERSION = 4
    PROTOCOL = "tcp"
    SOUR_IP = "10.0.1.0/24"
    SOUR_PORT = "8000"
    ID = None

    def setUp(self):
        super(TestFirewallRule, self).setUp()
        if not self.user_cloud._has_neutron_extension("fwaas_v2"):
            self.skipTest("fwaas_v2 service not supported by cloud")
        self.NAME = self.getUniqueString()
        sot = self.user_cloud.network.create_firewall_rule(
            name=self.NAME,
            action=self.ACTION,
            source_port=self.SOUR_PORT,
            destination_port=self.DEST_PORT,
            source_ip_address=self.SOUR_IP,
            destination_ip_address=self.DEST_IP,
            ip_version=self.IP_VERSION,
            protocol=self.PROTOCOL,
        )
        assert isinstance(sot, firewall_rule.FirewallRule)
        self.assertEqual(self.NAME, sot.name)
        self.ID = sot.id

    def tearDown(self):
        sot = self.user_cloud.network.delete_firewall_rule(
            self.ID, ignore_missing=False
        )
        self.assertIs(None, sot)
        super(TestFirewallRule, self).tearDown()

    def test_find(self):
        sot = self.user_cloud.network.find_firewall_rule(self.NAME)
        self.assertEqual(self.ID, sot.id)

    def test_get(self):
        sot = self.user_cloud.network.get_firewall_rule(self.ID)
        self.assertEqual(self.ID, sot.id)
        self.assertEqual(self.NAME, sot.name)
        self.assertEqual(self.ACTION, sot.action)
        self.assertEqual(self.DEST_IP, sot.destination_ip_address)
        self.assertEqual(self.DEST_PORT, sot.destination_port)
        self.assertEqual(self.IP_VERSION, sot.ip_version)
        self.assertEqual(self.SOUR_IP, sot.source_ip_address)
        self.assertEqual(self.SOUR_PORT, sot.source_port)

    def test_list(self):
        ids = [o.id for o in self.user_cloud.network.firewall_rules()]
        self.assertIn(self.ID, ids)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.functional.block_storage.v3 import base


class TestCapabilities(base.BaseBlockStorageTest):
    # getting capabilities can be slow
    TIMEOUT_SCALING_FACTOR = 1.5

    def test_get(self):
        services = list(self.operator_cloud.block_storage.services())
        host = [
            service
            for service in services
            if service.binary == 'cinder-volume'
        ][0].host

        sot = self.conn.block_storage.get_capabilities(host)
        self.assertIn('description', sot)
        self.assertIn('display_name', sot)
        self.assertIn('driver_version', sot)
        self.assertIn('namespace', sot)
        self.assertIn('pool_name', sot)
        self.assertIn('properties', sot)
        self.assertIn('replication_targets', sot)
        self.assertIn('storage_protocol', sot)
        self.assertIn('vendor_name', sot)
        self.assertIn('visibility', sot)
        self.assertIn('volume_backend_name', sot)



# Copyright 2019 Rackspace, US Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Amphora(resource.Resource):
    resource_key = 'amphora'
    resources_key = 'amphorae'
    base_path = '/octavia/amphorae'

    # capabilities
    allow_create = False
    allow_fetch = True
    allow_commit = False
    allow_delete = False
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id',
        'loadbalancer_id',
        'compute_id',
        'lb_network_ip',
        'vrrp_ip',
        'ha_ip',
        'vrrp_port_id',
        'ha_port_id',
        'cert_expiration',
        'cert_busy',
        'role',
        'status',
        'vrrp_interface',
        'vrrp_id',
        'vrrp_priority',
        'cached_zone',
        'created_at',
        'updated_at',
        'image_id',
        'image_id',
    )

    # Properties
    #: The ID of the amphora.
    id = resource.Body('id')
    #: The ID of the load balancer.
    loadbalancer_id = resource.Body('loadbalancer_id')
    #: The ID of the amphora resource in the compute system.
    compute_id = resource.Body('compute_id')
    #: The management IP of the amphora.
    lb_network_ip = resource.Body('lb_network_ip')
    #: The address of the vrrp port on the amphora.
    vrrp_ip = resource.Body('vrrp_ip')
    #: The IP address of the Virtual IP (VIP).
    ha_ip = resource.Body('ha_ip')
    #: The vrrp port's ID in the networking system.
    vrrp_port_id = resource.Body('vrrp_port_id')
    #: The ID of the Virtual IP (VIP) port.
    ha_port_id = resource.Body('ha_port_id')
    #: The date the certificate for the amphora expires.
    cert_expiration = resource.Body('cert_expiration')
    #: Whether the certificate is in the process of being replaced.
    cert_busy = resource.Body('cert_busy')
    #: The role configured for the amphora. One of STANDALONE, MASTER, BACKUP.
    role = resource.Body('role')
    #: The status of the amphora. One of: BOOTING, ALLOCATED, READY,
    #: PENDING_CREATE, PENDING_DELETE, DELETED, ERROR.
    status = resource.Body('status')
    #: The bound interface name of the vrrp port on the amphora.
    vrrp_interface = resource.Body('vrrp_interface')
    #: The vrrp group's ID for the amphora.
    vrrp_id = resource.Body('vrrp_id')
    #: The priority of the amphora in the vrrp group.
    vrrp_priority = resource.Body('vrrp_priority')
    #: The availability zone of a compute instance, cached at create time.
    cached_zone = resource.Body('cached_zone')
    #: The UTC date and timestamp when the resource was created.
    created_at = resource.Body('created_at')
    #: The UTC date and timestamp when the resource was last updated.
    updated_at = resource.Body('updated_at')
    #: The ID of the glance image used for the amphora.
    image_id = resource.Body('image_id')
    #: The ID of the compute flavor used for the amphora.
    compute_flavor = resource.Body('compute_flavor')


class AmphoraConfig(resource.Resource):
    base_path = '/octavia/amphorae/%(amphora_id)s/config'

    # capabilities
    allow_create = False
    allow_fetch = False
    allow_commit = True
    allow_delete = False
    allow_list = False
    allow_empty_commit = True

    requires_id = False

    # Properties
    #: The ID of the amphora.
    amphora_id = resource.URI('amphora_id')

    # The default _update code path also has no
    # way to pass has_body into this function, so overriding the method here.
    def commit(self, session, base_path=None):
        return super(AmphoraConfig, self).commit(
            session, base_path=base_path, has_body=False
        )


class AmphoraFailover(resource.Resource):
    base_path = '/octavia/amphorae/%(amphora_id)s/failover'

    # capabilities
    allow_create = False
    allow_fetch = False
    allow_commit = True
    allow_delete = False
    allow_list = False
    allow_empty_commit = True

    requires_id = False

    # Properties
    #: The ID of the amphora.
    amphora_id = resource.URI('amphora_id')

    # The default _update code path also has no
    # way to pass has_body into this function, so overriding the method here.
    def commit(self, session, base_path=None):
        return super(AmphoraFailover, self).commit(
            session, base_path=base_path, has_body=False
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.block_storage.v2 import stats as _stats
from openstack.tests.functional.block_storage.v2 import base


class TestStats(base.BaseBlockStorageTest):
    def setUp(self):
        super(TestStats, self).setUp()

        sot = self.operator_cloud.block_storage.backend_pools()
        for pool in sot:
            self.assertIsInstance(pool, _stats.Pools)

    def test_list(self):
        capList = [
            'volume_backend_name',
            'storage_protocol',
            'free_capacity_gb',
            'driver_version',
            'goodness_function',
            'QoS_support',
            'vendor_name',
            'pool_name',
            'thin_provisioning_support',
            'thick_provisioning_support',
            'timestamp',
            'max_over_subscription_ratio',
            'total_volumes',
            'total_capacity_gb',
            'filter_function',
            'multiattach',
            'provisioned_capacity_gb',
            'allocated_capacity_gb',
            'reserved_percentage',
            'location_info',
        ]
        capList.sort()
        pools = self.operator_cloud.block_storage.backend_pools()
        for pool in pools:
            caps = pool.capabilities
            keys = list(caps.keys())
            assert isinstance(caps, dict)
            # Check that we have at minimum listed capabilities
            for cap in sorted(capList):
                self.assertIn(cap, keys)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack import resource


class ShareExportLocation(resource.Resource):
    resource_key = "export_location"
    resources_key = "export_locations"
    base_path = "/shares/%(share_id)s/export_locations"

    # capabilities
    allow_list = True
    allow_fetch = True
    allow_create = False
    allow_commit = False
    allow_delete = False
    allow_head = False

    _max_microversion = '2.47'

    #: Properties
    # The share ID, part of the URI for export locations
    share_id = resource.URI("share_id", type='str')
    #: The path of the export location.
    path = resource.Body("path", type=str)
    #: Indicate if export location is preferred.
    is_preferred = resource.Body("preferred", type=bool)
    #: The share instance ID of the export location.
    share_instance_id = resource.Body("share_instance_id", type=str)
    #: Indicate if export location is admin only.
    is_admin = resource.Body("is_admin_only", type=bool)
    #: Indicate when the export location is created at
    created_at = resource.Body("created_at", type=str)
    #: Indicate when the export location is updated at
    updated_at = resource.Body("updated_at", type=str)
